import { useNavigate, useParams } from "@solidjs/router";
import { Component, Show, createEffect, createSignal, onMount } from "solid-js";

import styles from './GroupPage.module.css';
import { Group, Map, Profile, Strat, User, api } from "../api";
import { showDialog, showInputDialog, showMessageDialog } from "../state";
import { _arrayBufferToBase64, addMap, addMember, addStratType, errorToString, normalize } from "../util";
import { BsCheck2, BsPencil, BsTrash } from "solid-icons/bs";
import MemberItem from "../components/MemberItem";
import StratItem from "../components/StratItem";

const GroupPage: Component = () => {
	const params = useParams();
	const navigate = useNavigate();

	const [loading, setLoading] = createSignal(true);
	const [group, setGroup] = createSignal({} as Group);
	const [editing, setEditing] = createSignal(false);
	const [name, setName] = createSignal('');

	const [membersIsExpanded, setMembersIsExpanded] = createSignal(false);
	const [mapsIsExpanded, setMapsIsExpanded] = createSignal(false);
	const [stratsIsExpanded, setStratsIsExpanded] = createSignal(false);
	const [profilesIsExpanded, setProfilesIsExpanded] = createSignal(false);
	const [stratTypesIsExpanded, setStratTypesIsExpanded] = createSignal(false);

	const [filters, setFilters] = createSignal({
		name: "",
		Map: "All",
		stratType: "All"
	});

	onMount(async () => {
		try {
			setGroup(await api().getGroup(params.id));
			setName(group().name);
			setLoading(false);
		} catch (e) {
			showMessageDialog('Failed to load group', errorToString(e));
		}
	});

	const updateGroup = async (newValue: Group) => {
		try {
			await api().updateGroup(group().id, newValue);
			setGroup(newValue);
		} catch (e) {
			showMessageDialog('Failed to update group', errorToString(e));
		}
	};

	const deleteGroup = async () => {
		showDialog({
			title: 'Delete group',
			text: 'Do you want to delete this group?',
			buttons: [
				{
					name: 'Delete',
					type: 'danger',
					action: async () => {
						try {
							await api().removeGroup(group().id);
							navigate('/groups');
						} catch (e) {
							showMessageDialog('Failed to delete group', errorToString(e));
						}
					}
				},
				{
					name: 'Cancel',
					action: () => { }
				}
			]
		});
	};

	const createStrat = async () => {
		const [title, setTitle] = createSignal('');
		const [description, setDescription] = createSignal('');
		const [stratType, setStratType] = createSignal('');

		showDialog({
			title: 'Add a strat to Group',
			text: 'create a new strat',
			inputFields: [{
				onInput: setTitle,
				placeholder: 'title',
				name: 'title'
			}, {
				onInput: setDescription,
				placeholder: 'description',
				name: 'description'
			}],
			onDismiss: async () => {
				const newGroup = await api().addStrat(group().id, { title: title(), description: description(), stratType: stratType() });
				const strat = newGroup.strats.find(s => s.description === description() && s.title === title() && s.stratType === stratType());
				navigate('/group/' + group().id + '/strat/' + strat!.id);
			},
			content: <select name="stratType" id="stratType">
				{group().stratTypes.map(p => <option value={p} onclick={() => setStratType(p)}>{p}</option>)}
			</select>,
			buttons: ['confirm'],

		});
	}

	const toggleEdit = async () => {
		if (!editing()) {
			setEditing(true);
		} else {
			console.log(name());
			updateGroup({ ...group(), name: name() });
			setEditing(false);
		}
	};

	const onDeleteMember = async (member: User) => {
		setGroup({ ...group(), members: [...group().members.filter(m => m.username !== member.username)] });
	};

	const onDeleteMap = async (map: Map) => {
		setGroup({ ...group(), maps: [...group().maps.filter(m => m.id !== map.id)] });
	};

	const onDeleteStrat = async (strat: Strat) => {
		setGroup({ ...group(), strats: [...group().strats.filter(s => s.id !== strat.id)] });
	};

	const onDeleteStratType = async (stratType: string) => {
		setGroup({ ...group(), stratTypes: [...group().stratTypes.filter(s => s !== stratType)] });
	};

	const onDeleteProfile = async (profile: Profile) => {
		setGroup({ ...group(), profiles: [...group().profiles.filter(p => p.id !== profile.id)] });
	};

	const handleFilters = (filter: string, val: any) => {
		if (val === "Add") {
			switch (filter) {
				case "Map":
					addMap(group(), (g) => setGroup(g));
					break;
				case "StratType":
					addStratType(group(), (g) => setGroup(g));
				default:
					break;
			}
		}
		setFilters({ ...filters(), [filter]: val });
	}

	return (
		<>
			<Show when={!loading()} fallback={<div>Loading...</div>}>
				<div class={styles.groupName}>
					<span contentEditable={editing()} innerText={group().name} onInput={e => setName(normalize(e.currentTarget.innerText, 50, true))} />
					{editing() && <span class={styles.editCount}>{name().length}/50</span>}
				</div>
				<div class={styles.groupButtons}>
					<button class={styles.deleteButton} onClick={deleteGroup}>Delete group</button>
					<button class={styles.editButton} onClick={toggleEdit}>{editing() ? <BsCheck2 /> : <BsPencil />}</button>
				</div>
				<hr />
				<div class={styles.layout}>
					<div class={styles.strats}>
						<h3>Strats <button class={styles.addStratButton} onClick={() => createStrat()}>Add</button></h3>
						<div class={styles.filter}>
							Filter <br />
							<input name="name" type="text" placeholder="Strat Name"></input>
							<select name="Map" onInput={e => handleFilters(e.target.name, e.target.value)}>
								<option value={"All"} >Map</option>
								{group().maps.map(m => <option value={m.id}>{m.name}</option>)}
								<option value={"Add"} >Add</option>
							</select>
							<select name="StratType" onInput={e => handleFilters(e.target.name, e.target.value)}>
								<option value={"All"} >Strat Type</option>
								{group().stratTypes.map(s => <option value={s}>{s}</option>)}
								<option value={"Add"} >Add</option>
							</select>

						</div>
						{group().strats.map(s => <StratItem group={group()} onDelete={onDeleteStrat} strat={s} />)}
					</div>
					<div class={styles.members}>
						<h3>Member <button class={styles.addMemberButton} onClick={async () => addMember(group(), (g) => setGroup(g))}>Add</button></h3>
						{group().members.map(m => <MemberItem group={group()} member={m} onDelete={onDeleteMember} />)}
					</div>
				</div>
			</Show>
		</>
	)
}

export default GroupPage;
